@@ -38,6 +38,7 @@
 #include "main.h"
 #include "gthumb-window.h"
 #include "gtk-utils.h"
+#include "gconf-utils.h"
 
 
 #define GLADE_FILE "gthumb.glade"
@@ -76,6 +77,7 @@ new_catalog_cb (GtkWidget *widget,
 	char       *name_utf8, *name;
 	char       *path;
 	int         fd;
+	GtkTreeIter iter;
 
 	name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (data->window->app),
 					     GTK_DIALOG_MODAL,
@@ -100,12 +102,18 @@ new_catalog_cb (GtkWidget *widget,
 
 	fd = creat (path, 0644);
 	close (fd);
-	g_free (path);
 
 	sync ();
 
 	/* update the catalog list. */
+
 	catalog_list_change_to (data->cat_list, data->current_dir);
+
+	/* select the new catalog. */
+
+	if (catalog_list_get_iter_from_path (data->cat_list, path, &iter))
+		catalog_list_select_iter (data->cat_list, &iter);
+	g_free (path);
 }
 
 
@@ -155,6 +163,44 @@ new_dir_cb (GtkWidget *widget,
 /* -- "add to catalog" dialog -- */
 
 
+
+/* called when the "ok" button is clicked. */
+static void
+add_to_catalog__ok_cb (GtkWidget *widget, 
+		       DialogData *data)
+{
+	Catalog  *catalog;
+	gchar    *cat_path;
+	GList    *scan;
+	GError   *gerror;
+
+	cat_path = catalog_list_get_selected_path (data->cat_list);
+	if (cat_path == NULL)
+		return;
+
+	catalog = catalog_new ();
+	if (! catalog_load_from_disk (catalog, cat_path, &gerror)) {
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
+		return;
+	}
+
+	for (scan = data->data.list; scan; scan = scan->next)
+		catalog_add_item (catalog, (char*) scan->data);
+
+	if (! catalog_write_to_disk (catalog, &gerror)) 
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
+	else
+		all_windows_notify_cat_files_added (cat_path, data->data.list);
+
+	catalog_free (catalog);
+	
+	eel_gconf_set_path (PREF_ADD_TO_CATALOG_LAST_CATALOG, cat_path);
+	g_free (cat_path);
+
+	gtk_widget_destroy (data->dialog);
+}
+
+
 /* called when an item of the catalog list is activated. */
 static void 
 add_to_catalog__activated_cb (GtkTreeView *tree_view,
@@ -168,9 +214,14 @@ add_to_catalog__activated_cb (GtkTreeView *tree_view,
 	cat_path = catalog_list_get_path_from_tree_path (data->cat_list, path);
 	if (cat_path == NULL)
 		return;
-	catalog_list_change_to (data->cat_list, cat_path);
-	g_free (data->current_dir);
-	data->current_dir = cat_path;
+	
+	if (path_is_dir (cat_path)) {
+		catalog_list_change_to (data->cat_list, cat_path);
+		g_free (data->current_dir);
+		data->current_dir = cat_path;
+
+	} else if (path_is_file (cat_path)) 
+		add_to_catalog__ok_cb (NULL, data);
 }
 
 
@@ -204,40 +255,6 @@ add_to_catalog__destroy_cb (GtkWidget *widget,
 }
 
 
-/* called when the "ok" button is clicked. */
-static void
-add_to_catalog__ok_cb (GtkWidget *widget, 
-		       DialogData *data)
-{
-	Catalog  *catalog;
-	gchar    *cat_path;
-	GList    *scan;
-	GError   *gerror;
-
-	cat_path = catalog_list_get_selected_path (data->cat_list);
-	if (cat_path == NULL)
-		return;
-
-	catalog = catalog_new ();
-	if (! catalog_load_from_disk (catalog, cat_path, &gerror)) {
-		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
-		return;
-	}
-
-	for (scan = data->data.list; scan; scan = scan->next)
-		catalog_add_item (catalog, (char*) scan->data);
-
-	if (! catalog_write_to_disk (catalog, &gerror)) 
-		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
-	else
-		all_windows_notify_cat_files_added (cat_path, data->data.list);
-
-	catalog_free (catalog);
-	g_free (cat_path);
-	gtk_widget_destroy (data->dialog);
-}
-
-
 /* create the "add to catalog" dialog. */
 void
 dlg_add_to_catalog (GThumbWindow *window,
@@ -247,14 +264,20 @@ dlg_add_to_catalog (GThumbWindow *window,
 	DialogData       *data;
 	GtkWidget        *list_hbox;
 	GtkWidget        *cat_catalogs_label;
+	char             *last_catalog = NULL;
 
 	data = g_new (DialogData, 1);
 
 	data->window = window;
-	data->current_dir = get_catalog_full_path (NULL);
 	data->cat_list = catalog_list_new (FALSE);
 	data->data.list = list;
 
+	last_catalog = eel_gconf_get_path (PREF_ADD_TO_CATALOG_LAST_CATALOG, NULL);
+	if (path_is_file (last_catalog)) {
+		data->current_dir = remove_level_from_path (last_catalog);
+	} else
+		data->current_dir = get_catalog_full_path (NULL);
+
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE, NULL, NULL);
 	if (! data->gui) {
 		g_warning ("Could not find " GLADE_FILE "\n");
@@ -313,6 +336,13 @@ dlg_add_to_catalog (GThumbWindow *window,
 	catalog_list_change_to (data->cat_list, data->current_dir);
 	gtk_widget_set_sensitive (data->ok_btn, FALSE);
 
+	if (last_catalog != NULL) {
+		GtkTreeIter iter;
+		if (catalog_list_get_iter_from_path (data->cat_list, last_catalog, &iter))
+			catalog_list_select_iter (data->cat_list, &iter);
+		g_free (last_catalog);
+	}
+
 	/* run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
@@ -346,8 +376,8 @@ move_to_catalog_dir__activated_cb (GtkTreeView *tree_view,
 				   GtkTreeViewColumn *column,
 				   gpointer p)
 {
-	DialogData *  data = p;
-	gchar *       cat_path;
+	DialogData *data = p;
+	char       *cat_path;
 
 	cat_path = catalog_list_get_path_from_tree_path (data->cat_list, path);
 
@@ -364,10 +394,10 @@ move_to_catalog_dir__activated_cb (GtkTreeView *tree_view,
 
 /* called when the "ok" button is clicked. */
 static void
-move_to_catalog_dir__ok_cb (GtkWidget *widget, 
+move_to_catalog_dir__ok_cb (GtkWidget  *widget, 
 			    DialogData *data)
 {
-	gchar *new_path;
+	char *new_path;
 
 	new_path = g_strconcat (data->cat_list->path,
 				"/",
